@@ -17,32 +17,32 @@
 """Command-line interface to the OpenStack APIs"""
 
 import getpass
-import locale
 import logging
 import sys
 import traceback
 
 from cliff import app
 from cliff import command
+from cliff import commandmanager
 from cliff import complete
 from cliff import help
 from oslo_utils import importutils
 from oslo_utils import strutils
-import six
 
 from osc_lib.cli import client_config as cloud_config
 from osc_lib import clientmanager
-from osc_lib.command import commandmanager
 from osc_lib.command import timing
 from osc_lib import exceptions as exc
 from osc_lib.i18n import _
 from osc_lib import logs
 from osc_lib import utils
+from osc_lib import version
 
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
 
 DEFAULT_DOMAIN = 'default'
+DEFAULT_INTERFACE = 'public'
 
 
 def prompt_for_password(prompt=None):
@@ -153,14 +153,19 @@ class OpenStackShell(app.App):
 
     def close_profile(self):
         if self.do_profile:
-            trace_id = osprofiler_profiler.get().get_base_id()
+            profiler = osprofiler_profiler.get()
+            trace_id = profiler.get_base_id()
+            # Short ID for OpenTracing-based driver (64-bit id)
+            short_id = profiler.get_shorten_id(trace_id)
 
             # NOTE(dbelova): let's use warning log level to see these messages
             # printed. In fact we can define custom log level here with value
             # bigger than most big default one (CRITICAL) or something like
             # that (PROFILE = 60 for instance), but not sure we need it here.
             self.log.warning("Trace ID: %s" % trace_id)
-            self.log.warning("Display trace with command:\n"
+            self.log.warning("Short trace ID "
+                             "for OpenTracing-based drivers: %s" % short_id)
+            self.log.warning("Display trace data with command:\n"
                              "osprofiler trace show --html %s " % trace_id)
 
     def run_subcommand(self, argv):
@@ -249,10 +254,12 @@ class OpenStackShell(app.App):
             metavar='<interface>',
             dest='interface',
             choices=['admin', 'public', 'internal'],
-            default=utils.env('OS_INTERFACE'),
+            default=utils.env(
+                'OS_INTERFACE',
+                default=DEFAULT_INTERFACE),
             help=_('Select an interface type.'
                    ' Valid interface types: [admin, public, internal].'
-                   ' (Env: OS_INTERFACE)'),
+                   ' default=%s, (Env: OS_INTERFACE)') % DEFAULT_INTERFACE,
         )
         parser.add_argument(
             '--os-service-provider',
@@ -331,7 +338,7 @@ class OpenStackShell(app.App):
     * super()
     * _final_defaults()
     * OpenStackConfig
-    * get_one_cloud
+    * get_one
     * _load_plugins()
     * _load_commands()
     * ClientManager
@@ -389,7 +396,7 @@ class OpenStackShell(app.App):
         # Parent __init__ parses argv into self.options
         super(OpenStackShell, self).initialize_app(argv)
         self.log.info("START with options: %s",
-                      strutils.mask_password(self.command_options))
+                      strutils.mask_password(" ".join(self.command_options)))
         self.log.debug("options: %s",
                        strutils.mask_password(self.options))
 
@@ -397,10 +404,9 @@ class OpenStackShell(app.App):
         self._final_defaults()
 
         # Do configuration file handling
-        # Ignore the default value of interface. Only if it is set later
-        # will it be used.
         try:
             self.cloud_config = cloud_config.OSC_Config(
+                pw_func=prompt_for_password,
                 override_defaults={
                     'interface': None,
                     'auth_type': self._auth_type,
@@ -418,7 +424,7 @@ class OpenStackShell(app.App):
 
         # NOTE(dtroyer): Need to do this with validate=False to defer the
         #                auth plugin handling to ClientManager.setup_auth()
-        self.cloud = self.cloud_config.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=False,
@@ -457,7 +463,7 @@ class OpenStackShell(app.App):
         )
 
         # NOTE(dtroyer): If auth is not required for a command, skip
-        #                get_one_Cloud()'s validation to avoid loading plugins
+        #                get_one()'s validation to avoid loading plugins
         validate = cmd.auth_required
 
         # NOTE(dtroyer): Save the auth required state of the _current_ command
@@ -465,10 +471,13 @@ class OpenStackShell(app.App):
         self.client_manager._auth_required = cmd.auth_required
 
         # Validate auth options
-        self.cloud = self.cloud_config.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=validate,
+            app_name=self.client_manager._app_name,
+            app_version=self.client_manager._app_version,
+            additional_user_agent=[('osc-lib', version.version_string)],
         )
         # Push the updated args into ClientManager
         self.client_manager._cli_options = self.cloud
@@ -479,12 +488,21 @@ class OpenStackShell(app.App):
                 # let the command decide whether we need a scoped token
                 self.client_manager.validate_scope()
             # Trigger the Identity client to initialize
-            self.client_manager.auth_ref
+            self.client_manager.session.auth.auth_ref = \
+                self.client_manager.auth_ref
         return
 
     def clean_up(self, cmd, result, err):
         self.log.debug('clean_up %s: %s', cmd.__class__.__name__, err or '')
 
+        # Close SDK connection if available to have proper cleanup there
+        if hasattr(self.client_manager, "sdk_connection"):
+            self.client_manager.sdk_connection.close()
+
+        # Close session if available
+        if hasattr(self.client_manager.session, "session"):
+            self.client_manager.session.session.close()
+
         # Process collected timing data
         if self.options.timing:
             # Get session data
@@ -511,12 +529,6 @@ class OpenStackShell(app.App):
 def main(argv=None):
     if argv is None:
         argv = sys.argv[1:]
-        if six.PY2:
-            # Emulate Py3, decode argv into Unicode based on locale so that
-            # commands always see arguments as text instead of binary data
-            encoding = locale.getpreferredencoding()
-            if encoding:
-                argv = map(lambda arg: arg.decode(encoding), argv)
     return OpenStackShell().run(argv)
 
 
