@@ -19,7 +19,6 @@ import copy
 import getpass
 import logging
 import os
-import six
 import time
 import warnings
 
@@ -51,7 +50,7 @@ def backward_compat_col_lister(column_headers, columns, column_map):
     #                volume v1, convert it to a list in order to change
     #                the column name.
     column_headers = list(column_headers)
-    for old_col, new_col in six.iteritems(column_map):
+    for old_col, new_col in column_map.items():
         if old_col in columns:
             LOG.warning(_('The column "%(old_column)s" was deprecated, '
                           'please use "%(new_column)s" replace.') % {
@@ -79,7 +78,7 @@ def backward_compat_col_showone(show_object, columns, column_map):
         return show_object
 
     show_object = copy.deepcopy(show_object)
-    for old_col, new_col in six.iteritems(column_map):
+    for old_col, new_col in column_map.items():
         if old_col in columns:
             LOG.warning(_('The column "%(old_column)s" was deprecated, '
                           'please use "%(new_column)s" replace.') % {
@@ -284,10 +283,11 @@ def find_resource(manager, name_or_id, **kwargs):
         raise exceptions.CommandError(msg % name_or_id)
 
 
-def format_dict(data):
+def format_dict(data, prefix=None):
     """Return a formatted string of key value pairs
 
     :param data: a dict
+    :param prefix: the current parent keys in a recursive call
     :rtype: a string formatted to key='value'
     """
 
@@ -296,7 +296,18 @@ def format_dict(data):
 
     output = ""
     for s in sorted(data):
-        output = output + s + "='" + six.text_type(data[s]) + "', "
+        if prefix:
+            key_str = ".".join([prefix, s])
+        else:
+            key_str = s
+        if isinstance(data[s], dict):
+            # NOTE(dtroyer): Only append the separator chars here, quoting
+            #                is completely handled in the terminal case.
+            output = output + format_dict(data[s], prefix=key_str) + ", "
+        elif data[s] is not None:
+            output = output + key_str + "='" + str(data[s]) + "', "
+        else:
+            output = output + key_str + "=, "
     return output[:-2]
 
 
@@ -692,3 +703,55 @@ def wait_for_status(status_f,
             callback(progress)
         time.sleep(sleep_time)
     return retval
+
+
+def get_osc_show_columns_for_sdk_resource(
+    sdk_resource,
+    osc_column_map,
+    invisible_columns=None
+):
+    """Get and filter the display and attribute columns for an SDK resource.
+
+    Common utility function for preparing the output of an OSC show command.
+    Some of the columns may need to get renamed, others made invisible.
+
+    :param sdk_resource: An SDK resource
+    :param osc_column_map: A hash of mappings for display column names
+    :param invisible_columns: A list of invisible column names
+
+    :returns: Two tuples containing the names of the display and attribute
+              columns
+    """
+
+    if getattr(sdk_resource, 'allow_fetch', None) is not None:
+        # OSC at the moment lands here with FakeResource objects which are not
+        # 100% sdk compatible. Unless we introduce SDK test/fake resources we
+        # should check presence of the specific method
+        resource_dict = sdk_resource.to_dict(
+            body=True, headers=False, ignore_none=False)
+    else:
+        # We might land here with not a real SDK Resource (during the
+        # transition period).
+        resource_dict = sdk_resource
+
+    # Build the OSC column names to display for the SDK resource.
+    attr_map = {}
+    display_columns = list(resource_dict.keys())
+    invisible_columns = [] if invisible_columns is None else invisible_columns
+    for col_name in invisible_columns:
+        if col_name in display_columns:
+            display_columns.remove(col_name)
+    for sdk_attr, osc_attr in osc_column_map.items():
+        if sdk_attr in display_columns:
+            attr_map[osc_attr] = sdk_attr
+            display_columns.remove(sdk_attr)
+        if osc_attr not in display_columns:
+            display_columns.append(osc_attr)
+    sorted_display_columns = sorted(display_columns)
+
+    # Build the SDK attribute names for the OSC column names.
+    attr_columns = []
+    for column in sorted_display_columns:
+        new_column = attr_map[column] if column in attr_map else column
+        attr_columns.append(new_column)
+    return tuple(sorted_display_columns), tuple(attr_columns)
